;************************************************
;*	USB 78K0 Tester Program (uPD78F0730)	*
;************************************************
;V096:Add SW Chattering,Add LCD Initial,AnalogOut 15~620Hz,Auto Mode Display 5 Wait
;V097:PlusOut CR000 Set Bug Fix,SW Chattering Time 100ms --> 10ms,
;    :Auto Mode Display 5 --> 2 Wait,AMCNT Clear Bug Fix

;****************************************
;*	RAM Definition			*
;****************************************
RAM	DSEG	AT	0F000H
VIDAT:	DS	2			;V,I Data
RDAT:	DS	2			;R Data
CCNT:	DS	2			;C 40us Counter
OUTF:	DS	2			;Pulse Out f
O1DATT:	DS	2			;Osc Ch1 Data
DSPDAT:	DS	2			;Hex Display Data
OUTAF:	DS	2			;D/A Out f
TMPADD:	DS	2			;Address Temporary
ADADD:	DS	2			;A/D Data Address
TLVLAD:	DS	2			;Trig Level(Compare A/D)

ADDADD:	DS	2			;A/D Data Address
O1DADD:	DS	2			;Osc Ch1 Data Address
ADDATR:	DS	2			;A/D Data (Roll)

ADDAT:	DS	146			;A/D Data

DSPD1:	DS	1			;Display Data 1
DSPD2:	DS	1			;Display Data 2
DSPD3:	DS	1			;Display Data 3
DSPD4:	DS	1			;Display Data 4

MODE:	DS	1			;Mode
DACNT:	DS	1			;D/A Output Data Address Counter
SWDAT:	DS	1			;SW Data
PCPOS:	DS	1			;Pulse Mode Cursor Position
ACPOS:	DS	1			;D/A Mode Cursor Position
AMODE:	DS	1			;D/A Out Mode

O1DAT:	DS	73			;Osc ch1 Data
PMAX:	DS	1			;Plot Max Position
PMIN:	DS	1			;Plot Min Position
POLD:	DS	1			;Plot Old Data
PDAT:	DS	4			;Plot Data
TRGFLG:	DS	1			;Trig Flag
BTRGFLG:DS	1			;Before Trig Flag

VRANG:	DS	1			;Volt Range
TMODE:	DS	1			;Trig Mode
DMODE:	DS	1			;Dousa Mode
TRANG:	DS	1			;Time Range
TLVL:	DS	1			;Trig Level
OCPOS:	DS	1			;Osc Mode Cursor Position
DSPFLG:	DS	1			;Osc Mode Display Flag

AINTCT:	DS	1			;A/D Interupt Counter
AINTCP:	DS	1			;A/D Interupt Counter Comparete
AMCNT:	DS	1			;Auto Mode Counter

VRAM:	DS	488			;Virtual VRAM

;****************************************
;*	      Initial Routine		*
;****************************************
;ROM	CSEG	AT	0F000H		;RAM
ROM	CSEG	AT	02000H		;ROM

START:	MOVW	HL,#DAINT		;Timer H1 Interupt Routine Registe
	PUSH	HL
	MOVW	AX,#9
	CALL	!18E0H
	POP	HL

	MOVW	HL,#ADINT		;16Bit Timer Interupt Routine Registe
	PUSH	HL
	MOVW	AX,#11
	CALL	!18E0H
	POP	HL

	MOV	P1,#00101001B		;Port Data Set (P1)
	MOV	P3,#00001001B		;	P3
	MOV	P6,#00000010B		;	P6
	MOV	P12,#00000001B		;	P12

	MOV	PM0,#11111101B		;Port Mode Set P0(0:Output,1:Input)
	MOV	PM1,#00010010B		;	P1
	MOV	PM3,#11110000B		;	P3
	MOV	PM6,#11111100B		;	P6
	MOV	PM12,#11111110B		;	P12

	MOV	PU1,#00000010B		;Port PullUp Set P0(0:Off,1:On)

	SET1	CSIM10.6		;CSI10 Tx Mode Set

	MOV	TCL50,#00000101B	;Timer50 Set(2kHz Output Set)
	MOV	CR50,#61		;	2kHz Set
	MOV	TMC50,#00000111B	;	Plus Out Set
	SET1	TMC50.7			;Timer50 Start

	MOV	TCL51,#00000100B	;Timer51 Set(40us Interval Set)
	MOV	CR51,#40		;	40us Set
	SET1	TMC51.7			;Timer51 Start

	MOV	CRC00,#00000100B	;16Bit Timer Set(CR000:CMP,CR010:CAP Set)
	MOV	TOC00,#00000010B	;	CR000 H <--> L On / TO000 Disable
	MOVW	CR000,#00001H		;	TO000 Output 8MHz Set

	SET1	TMHMD1.4		;TimerH1 25us Interupt Set
	MOV	CMP01,#99
	CLR1	MK0H.3			;	Interupt Mask Clear

	CLR1	P3.3			;LCD Initial(CS1 L Set)
	CLR1	P1.5			;	CS2 L Set
	CLR1	P1.6			;	Data/Instruction I Set
	MOV	A,#0E2H			;	Reset
	CALL	!SROUT
	MOV	A,#0A0H			;	ADC True
	CALL	!SROUT
	MOV	A,#0A4H			;	Statics Drive Off
	CALL	!SROUT
	MOV	A,#0A9H			;	Duty 1/32
	CALL	!SROUT
	SET1	P1.6			;	Data/Instruction D Set
	SET1	P3.3			;	CS1 H Set
	SET1	P1.5			;	CS2 H Set

	MOV	A,#0			;Initial Data Set
	MOV	!MODE,A
	MOV	!PCPOS,A
	MOV	!ACPOS,A
	MOV	!AMODE,A
	MOV	!VRANG,A
	MOV	!TMODE,A
	MOV	!DMODE,A
	MOV	!TRANG,A
	MOV	!OCPOS,A
	MOV	!DSPFLG,A
	MOV	!AMCNT,A

	MOV	A,#9
	MOV	!TLVL,A

	MOVW	AX,#620
	MOVW	!OUTAF,AX
	MOVW	AX,#4000
	MOVW	!OUTF,AX

	EI				;Interrupt Enable

;****************************************
;*	      Main Routine		*
;****************************************
MAIN:	CALL	!LCDCL			;LCD ALL CLEAR
	CALL	!MENDSP			;Menu Display
	CALL	!VLCD			;VRAM --> LCD

MAN010:	CALL	!SWREAD			;SW Read
	MOV	A,!SWDAT

	CMP	A,#1			;SW1 On ?
	BNZ	$MAN030			;	(No)

	MOV	A,#0FFH			;MODE <-- MODE - 1
	ADD	A,!MODE
	CMP	A,#0FFH			;	MODE > 7 ?
	BNZ	$MAN020			;		(No)
	MOV	A,#7
MAN020:	MOV	!MODE,A
	BR	MAIN

MAN030:	CMP	A,#2			;SW2 On ?
	BNZ	$MAN050			;	(No)

	MOV	A,#1			;MODE <-- MODE + 1
	ADD	A,!MODE
	CMP	A,#8			;	MODE > 7 ?
	BC	$MAN040			;		(No)
	MOV	A,#0
MAN040:	MOV	!MODE,A
	BR	MAIN

MAN050:	CMP	A,#3			;SW3 On ?
	BNZ	$MAN010			;	(No)

	MOV	A,!MODE

	CMP	A,#0			;;MODE = 0 ?
	BNZ	$MAN051			;	(No)
	CALL	!OMODE			;	Osc1 Mode
	BR	MAIN

MAN051:	CMP	A,#1			;;MODE = 1 ?
	BNZ	$MAN052			;	(No)
	CALL	!DAMODE			;	AnalogOut Mode
	BR	MAIN

MAN052:	CMP	A,#2			;MODE = 2 ?
	BNZ	$MAN053			;	(No)
	CALL	!VMODE			;	V Mode
	BR	MAIN

MAN053:	CMP	A,#3			;;MODE = 3 ?
	BNZ	$MAN054			;	(No)
	CALL	!IMODE			;	I Mode
	BR	MAIN

MAN054:	CMP	A,#4			;;MODE = 4 ?
	BNZ	$MAN055			;	(No)
	CALL	!RMODE			;	R Mode
	BR	MAIN

MAN055:	CMP	A,#5			;;MODE = 5 ?
	BNZ	$MAN056			;	(No)
	CALL	!CMODE			;	C Mode
	BR	MAIN

MAN056:	CMP	A,#6			;;MODE = 6 ?
	BNZ	$MAN057			;	(No)
	CALL	!FMODE			;	F Mode
	BR	MAIN

MAN057:	CMP	A,#7			;;MODE = 7 ?
	BNZ	$MAN059			;	(No)
	CALL	!PLMODE			;	PlusOut Mode
	BR	MAIN

MAN059:	BR	MAIN

;****************************************
;*	      Sub Routine		*
;****************************************
;************** Osc Mode ****************
OMODE:	MOVW	AX,#ADDAT		;Initial Data Set
	MOVW	!ADADD,AX
	MOV	A,#0H
	MOV	!TRGFLG,A		;Trig Flag Clear
	MOV	!DSPFLG,A		;Display Flag Clear
	MOV	A,#0FFH
	MOV	!BTRGFLG,A		;Before Trig Flag Set

	SET1	PRM00.0			;16Bit Timer Clock 4MHz Set
	MOV	TMC00,#00001100B	;16Bit Timer Start
	CALL	!CRSET			;CR000 Data Set
	CLR1	MK0H.6			;16Bit Timer Interupt Mask Clear

	CALL	!LCDCL			;LCD ALL CLEAR
	CALL	!WAKDSP			;Waku Display

OMD000:	CALL	!OMNDSP			;Menu Display
	CALL	!OTPDSP			;Trig Level Display
	CALL	!OPOSDP			;Position Marker Display
	CALL	!OMENIV			;Cursol Display
	CALL	!VLCD			;VRAM --> LCD
	CALL	!TLSET			;Trig Level Set
	CALL	!CRSET			;CR000 Data Set

OMD010:	MOV	A,!DSPFLG		;Display Flag 1 ?
	BT	A.0,$OMD100		;	(Yes)
	BR	OMD900

OMD100:	MOV	A,!DMODE		;Run ?
	BF	A.0,$OMD105		;	(Yes)
	BR	OMD190

OMD105:	CALL	!WAKDSP			;Waku Display
	MOV	A,!TMODE		;Roll Modet ?
	BF	A.1,$OMD107		;	(No)

	MOVW	HL,#ADDAT		;ADDAT <-- ADDAT + 2
	MOV	B,#144
OMD106:	MOV	A,[HL+2]
	MOV	[HL],A
	INCW	HL
	DBNZ	B,$OMD106

	MOVW	AX,!ADDATR		;A/D Data Set(Roll)
	MOV	[HL+1],A
	MOV	A,X
	MOV	[HL],A

OMD107:	MOVW	AX,#ADDAT		;ADDAT --> O1DAT
	MOVW	!ADDADD,AX		;	ADDADD <-- ADDAT Addresst
	MOVW	AX,#O1DAT
	MOVW	!O1DADD,AX		;	O1DADD <-- O1DAT Address

OMD110:	MOVW	AX,!ADDADD		;	DE <-- [ADDAT]
	MOVW	HL,AX
	MOV	A,[HL]			;
	MOV	X,A
	MOV	A,[HL+1]
	MOVW	DE,AX

	MOV	A,!VRANG		;	HL <-- TLTBL + VRANG * 4
	MOV	X,#4			;		A <-- VRANG * 4
	MULU	X
	MOV	A,X

	MOVW	HL,#TLTBL		;		HL <-- HL + A
	ADD	L,A
	MOV	A,#0
	ADDC	H,A

	MOV	A,[HL]			;	DE <-- (ADDAT - [HL+1][HL]) * [HL+2] / [HL+3]
	SUB	E,A			;		DE <-- (ADDAT - [HL+1][HL])
	MOV	A,[HL+1]
	SUBC	D,A
	BNC	$OMD120
	MOVW	DE,#0

OMD120:	MOV	A,[HL+2]		;		C <-- [HL+2]
	MOV	C,A

	MOVW	AX,DE			;		DE <-- (ADDAT - [HL+1][HL]) * [HL+2]
	MOVW	!O1DATT,AX		;
	XCH	A,X
	MOV	A,C			;			AX <-- O1DATT * [HL+2]
	MULU	X			;				B <-- O1DATT(H) * C
	MOV	A,X
	MOV	B,A
	MOVW	AX,!O1DATT		;				AX <-- O1DATT(L) * C
	MOV	A,C
	MULU	X
	ADD	A,B			;				AX <-- B * H'100 + AX
	MOVW	DE,AX

	MOV	A,[HL+3]		;		DE <-- (ADDAT - [HL+1][HL]) * [HL+2] / [HL+3]
	MOV	C,A
	MOVW	AX,DE
	DIVUW	C
	MOVW	DE,AX

	MOVW	AX,!O1DADD		;	O1DAT Data Set
	MOVW	HL,AX
	MOV	A,E
	MOV	[HL],A

	MOVW	AX,!ADDADD		;	ADDADD + 2
	INCW	AX
	INCW	AX
	MOVW	!ADDADD,AX

	MOVW	AX,!O1DADD		;	O1DADD + 1
	INCW	AX
	MOVW	!O1DADD,AX
	CMPW	AX,#O1DAT+73
	BC	$OMD110

	MOV	A,#0			;POLD Initial Data Set
	MOV	!POLD,A
	MOV	B,#0			;B = 0 ~ 72 Loop

OMD130:	MOV	A,!POLD			;	PMIN <-- POLD
	MOV	!PMIN,A
	MOVW	HL,#O1DAT		;	PMAX <-- A/D Data
	MOV	A,[HL+B]
	MOV	!PMAX,A
	MOV	!POLD,A			;	POLD <-- A/D Data

	CMP	A,!PMIN			;	PMAX,PMIN Set
	BNC	$OMD140
	XCH	A,!PMIN
	XCH	A,!PMAX

OMD140:	MOV	D,#002H			;	D <-- Dot Set Mask Data Set
	MOV	A,#0			;	PDAT Clear
	MOV	!PDAT+0,A
	MOV	!PDAT+1,A
	MOV	!PDAT+2,A
	MOV	!PDAT+3,A
	MOV	X,#0			;	PDAT Counter 0 Set
	MOV	E,#0			;	E = 0 ~30 Loop

OMD150:	MOV	A,E			;		PMIN < E < PMAX ?
	CMP	A,!PMIN
	BZ	$OMD160
	BC	$OMD170			;			(No)
	CMP	A,!PMAX
	BZ	$OMD160
	BNC	$OMD170			;			(No)

OMD160:	MOVW	HL,#PDAT		;		HL <-- PDAT Address Set
	MOV	A,X			;		C <-- X PDAT Counter:(0 ~ 3)
	MOV	C,A
	MOV	A,D			;		Dot Mask Data Set
	OR	A,[HL+C]
	MOV	[HL+C],A

OMD170:	MOV	A,D			;		Dot Mask Data Left Rotate
	ROL	A,1
	MOV	D,A
	BNC	$OMD180			;		PDAT Address Next ? (No)
	INC	X			;		PDAT Counter(X) <-- PDAT Counter(X) + 1

OMD180:	INC	E			;		E <-- E + 1
	MOV	A,E
	CMP	A,#31
	BC	$OMD150			;	E Loop End

	MOVW	HL,#VRAM+49+0		;VRAM Set
	MOV	A,!PDAT+0
	OR	A,[HL+B]
	MOV	[HL+B],A

	MOVW	HL,#VRAM+49+122		;VRAM Set
	MOV	A,!PDAT+1
	OR	A,[HL+B]
	MOV	[HL+B],A

	MOVW	HL,#VRAM+49+244		;VRAM Set
	MOV	A,!PDAT+2
	OR	A,[HL+B]
	MOV	[HL+B],A

	MOVW	HL,#VRAM+49+366		;VRAM Set
	MOV	A,!PDAT+3
	OR	A,[HL+B]
	MOV	[HL+B],A

	INC	B
	MOV	A,B
	CMP	A,#73
	BC	$OMD130			;B Loop End

	CALL	!VLCD			;VRAM --> LCD
OMD190:	MOV	A,#0H
	MOV	!TRGFLG,A		;Trig Flag Clear
	MOV	!DSPFLG,A		;Display Flag Clear
	MOV	!AMCNT,A		;Auto Mode Counter Clear
	MOV	A,#0FFH
	MOV	!BTRGFLG,A		;Before Trig Flag Set
	CLR1	MK0H.6			;16Bit Timer Interupt Mask Clear

OMD900:	CALL	!SWREAD			;SW Read
	MOV	A,!SWDAT
	CMP	A,#1			;SW1 On ?
	BNZ	$OMD920			;	(No)

	MOV	A,!OCPOS		;	OCPOS <-- OCPOS + 1
	INC	A
	CMP	A,#5
	BC	$OMD910
	MOV	A,#0
OMD910:	MOV	!OCPOS,A
	BR	OMD000

OMD920:	CMP	A,#2			;SW2 On ?
	BNZ	$OMD940			;	(No)

	MOV	A,!OCPOS
	CMP	A,#0			;OCPOS = 0 ?
	BNZ	$OMD922

	MOV	A,!VRANG
	ADD	A,#1
	CMP	A,#6
	BC	$OMD921
	MOV	A,#5
OMD921:	MOV	!VRANG,A
	BR	OMD000

OMD922:	CMP	A,#1			;OCPOS = 1 ?
	BNZ	$OMD924

	MOV	A,!TRANG
	CMP	A,#8
	BNC	$OMD923

	MOV	A,!TMODE
	XOR	A,#00000001B
	MOV	!TMODE,A
OMD923:	BR	OMD000

OMD924:	CMP	A,#2			;OCPOS = 2 ?
	BNZ	$OMD926

	MOV	A,!DMODE
	XOR	A,#00000001B
	MOV	!DMODE,A
	BR	OMD000

OMD926:	CMP	A,#3			;OCPOS = 3 ?
	BNZ	$OMD928

	MOV	A,!TRANG
	ADD	A,#1
	CMP	A,#11
	BC	$OMD927
	MOV	A,#10
OMD927:	MOV	!TRANG,A
	CMP	A,#8
	BC	$OMD927A
	MOV	A,#2
	MOV	!TMODE,A
OMD927A:BR	OMD000

OMD928:	MOV	A,!TLVL			;OCPOS = 4
	SUB	A,#1
	BNC	$OMD929
	MOV	A,#0
OMD929:	MOV	!TLVL,A
	BR	OMD000

OMD940:	CMP	A,#3			;SW3 On ?
	BNZ	$OMD960			;	(No)

	MOV	A,!OCPOS
	CMP	A,#0			;OCPOS = 0 ?
	BNZ	$OMD942

	MOV	A,!VRANG
	SUB	A,#1
	BNC	$OMD941
	MOV	A,#0
OMD941:	MOV	!VRANG,A
	BR	OMD000

OMD942:	CMP	A,#1			;OCPOS = 1 ?
	BNZ	$OMD944

	MOV	A,!TRANG
	CMP	A,#8
	BNC	$OMD943

	MOV	A,!TMODE
	XOR	A,#00000001B
	MOV	!TMODE,A
OMD943:	BR	OMD000

OMD944:	CMP	A,#2			;OCPOS = 2 ?
	BNZ	$OMD946

	MOV	A,!DMODE
	XOR	A,#00000001B
	MOV	!DMODE,A
	BR	OMD000

OMD946:	CMP	A,#3			;OCPOS = 3 ?
	BNZ	$OMD948

	MOV	A,!TRANG
	SUB	A,#1
	BNC	$OMD947
	MOV	A,#0
OMD947:	MOV	!TRANG,A
	CMP	A,#7
	BNZ	$OMD947A
	MOV	A,#0
	MOV	!TMODE,A
OMD947A:BR	OMD000

OMD948:	MOV	A,!TLVL			;OCPOS = 4
	ADD	A,#1
	CMP	A,#31
	BC	$OMD949
	MOV	A,#30
OMD949:	MOV	!TLVL,A
	BR	OMD000

OMD960:	CMP	A,#4			;Long SW3 On ?
	BZ	$OMD999			;	(Yes)
	BR	OMD010			;	(No)

OMD999:	MOV	TMC00,#00000000B	;16Bit Timer Stop
	CLR1	PRM00.0			;16Bit Timer Clock 8MHz Set
	SET1	MK0H.6			;16Bit Timer Interupt Mask Set
	RET

;********  16Bit Timer Interupt *********
ADINT:	PUSH	PSW			;Register Push
	PUSH	HL
	PUSH	AX
	PUSH	BC
	PUSH	DE

	MOV	A,!AINTCT		;AINTCNT = AINTCNT + 1
	INC	A
	MOV	!AINTCT,A
	CMP	A,!AINTCP		;AINTCT = AINTCP ?
	BNC	$ADI000			;	(Yes)
	BR	ADI090			;	(No)

ADI000:	MOV	A,#0			;AINTCT Clear
	MOV	!AINTCT,A

	MOV	A,#10111111B		;A/D Data Set(Osc ch1 Read)
	CALL	!ADREAD			;	A/D ch6,Single End,Unipolar Set
	MOVW	DE,AX			;	DE <-- A/D Data
	MOVW	!ADDATR,AX

	MOV	A,!TMODE		;Roll Modet ?
	BT	A.1,$ADI035		;	(Yes)

	MOVW	AX,!ADADD		;	[ADADD] <-- A/D Data
	MOVW	HL,AX
	MOV	A,E
	MOV	[HL],A
	MOV	A,D
	MOV	[HL+1],A

	MOV	A,!TRGFLG		;Trig Flag Set ?
	BT	A.0,$ADI020		;	(Yes)

	MOV	A,!BTRGFLG		;B <-- Before TRGFLG
	MOV	B,A

	MOV	A,!TLVLAD		;Trig ?
	SUB	A,E
	MOV	A,!TLVLAD+1
	SUBC	A,D
	BC	$ADI010			;	(No)

	MOV	A,#0FFH			;Before TRGFLG Set
	MOV	!BTRGFLG,A

	MOV	A,B			;Before TRGFLG Set ?
	BT	A.0,$ADI020		;	(Yes)

	MOV	A,#0FFH			;Trig Flag Set
	MOV	!TRGFLG,A
	BR	ADI040

ADI010:	MOV	A,#000H			;Before TRGFLG Clear
	MOV	!BTRGFLG,A

ADI020:	MOVW	AX,!ADADD		;ADADD > [ADDAT] + 146 ?
	INCW	AX
	INCW	AX
	MOVW	!ADADD,AX
	CMPW	AX,#ADDAT+146
	BC	$ADI090			;	(No)

	MOV	A,!TRGFLG		;Trig Flag Set ?
	BT	A.0,$ADI030		;	(Yes)

	MOV	A,!TMODE		;Auto Mode ?
	BT	A.0,$ADI040		;	(No)

	MOV	A,!AMCNT		;Auto Mode Counter + 1
	INC	A
	MOV	!AMCNT,A
	CMP	A,#2			;Auto Mode Counter > 2 ?
	BC	$ADI040			;	(No)
	MOV	A,#0			;Auto Mode Counter Clear
	MOV	!AMCNT,A

ADI030:	SET1	MK0H.6			;16Bit Timer Interupt Mask Set
ADI035:	MOV	A,#0FFH			;Display Flag Set
	MOV	!DSPFLG,A

ADI040:	MOVW	AX,#ADDAT		;ADADD Initial Data Set
	MOVW	!ADADD,AX

ADI090:	POP	DE			;Register Pop
	POP	BC
	POP	AX
	POP	HL
	POP	PSW

	RET

;************* Trig Level Set ***********
TLSET:	MOV	A,!VRANG		;HL <-- TLTBL + VRANG * 4
	MOV	X,#4			;	A <-- VRANG * 4
	MULU	X
	MOV	A,X

	MOVW	HL,#TLTBL		;	HL <-- HL + A
	ADD	L,A
	MOV	A,#0
	ADDC	H,A

	MOV	A,!TLVL			;DE <-- TLVL * [HL+3] / [HL+2] + [HL+1][HL]
	MOV	X,A			;	DE <-- TLVL * [HL+3]
	MOV	A,[HL+3]
	MULU	X
	MOVW	DE,AX

	MOV	A,[HL+2]		;	C <-- [HL+2]
	MOV	C,A

	MOVW	AX,DE			;	DE <-- DE / C
	DIVUW	C
	MOVW	DE,AX

	MOV	A,[HL]			;	DE <-- DE + [HL+1][HL]
	ADD	E,A
	MOV	A,[HL+1]
	ADDC	D,A

	MOVW	AX,DE
	MOVW	!TLVLAD,AX

	RET


TLTBL:	DB	0F6H,001H,5,3,0EEH,001H,15,17,0D4H,001H,15,43
	DB	0AAH,001H,3,17,055H,001H,3,34,056H,000H,3,85

;************* CR000 Set ****************
CRSET:	MOVW	HL,#CRTBL-2
	MOV	A,!TRANG
	MOV	B,A
	INC	B
CRS010:	INCW	HL
	INCW	HL
	DBNZ	B,$CRS010
	MOV	A,[HL]
	MOV	X,A
	MOV	A,[HL+1]
	MOVW	CR000,AX

	MOVW	HL,#CPTBL-1
	MOV	A,!TRANG
	MOV	B,A
	INC	B
CRS020:	INCW	HL
	DBNZ	B,$CRS020
	MOV	A,[HL]
	MOV	!AINTCP,A

	RET

CRTBL:	DW	333,666,1333,3333,6666,13333,33333,33333,33333,33333,33333

CPTBL:	DB	1,1,1,1,1,1,1,2,4,10,20

;*********** AnalogOut Mode *************
DAMODE:	SET1	TMHMD1.7		;TimerH1 Start
	CALL	!LCDCL			;LCD ALL CLEAR

	MOV	A,#17H			;h Display
	MOV	E,#93			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,#18H			;z Display
	MOV	E,#108			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	CALL	!PMNDSP			;Menu Display

DMD010:	MOVW	AX,!OUTAF		;1~4 Keta Display
	CALL	!DSP40
	CALL	!AMDSP			;Mode Display
	CALL	!AMENIV			;Cursol Invert
	CALL	!VLCD			;VRAM --> LCD

DMD050:	CALL	!SWREAD			;SW Read
	MOV	A,!SWDAT
	CMP	A,#1			;	SW1 On ?
	BNZ	$DMD070			;		(No)

	MOV	A,!ACPOS		;ACPOS <-- ACPOS + 1
	INC	A
	AND	A,#00000011B
	MOV	!ACPOS,A
	BR	DMD010

DMD070:	CMP	A,#2			;	SW2 On ?
	BNZ	$DMD080			;		(No)

	MOV	A,!ACPOS
	CMP	A,#0
	BNZ	$DMD072

	MOV	A,!AMODE
	ADD	A,#1
	CMP	A,#3
	BC	$DMD071
	MOV	A,#0
DMD071:	MOV	!AMODE,A
	BR	DMD010

DMD072:	CMP	A,#1
	BNZ	$DMD074
	MOVW	AX,!OUTAF
	ADDW	AX,#100
	BR	DMD078

DMD074:	CMP	A,#2
	BNZ	$DMD076
	MOVW	AX,!OUTAF
	ADDW	AX,#10
	BR	DMD078

DMD076:	MOVW	AX,!OUTAF
	ADDW	AX,#1

DMD078:	CMPW	AX,#621
	BC	$DMD089
	MOVW	AX,#620
	BR	DMD089

DMD080:	CMP	A,#3			;	SW3 On ?
	BZ	$DMD80A			;		(Yes)
	BR	DMD090			;		(No)

DMD80A:	MOV	A,!ACPOS
	CMP	A,#0
	BNZ	$DMD082

	MOV	A,!AMODE
	SUB	A,#1
	CMP	A,#0FFH
	BNZ	$DMD081
	MOV	A,#2
DMD081:	MOV	!AMODE,A
	BR	DMD010

DMD082:	CMP	A,#1
	BNZ	$DMD084
	MOVW	AX,!OUTAF
	SUBW	AX,#100
	BR	DMD088

DMD084:	CMP	A,#2
	BNZ	$DMD086
	MOVW	AX,!OUTAF
	SUBW	AX,#10
	BR	DMD088

DMD086:	MOVW	AX,!OUTAF
	SUBW	AX,#1

DMD088:	BC	$DMD88A
	BNZ	$DMD089
DMD88A:	MOVW	AX,#15

DMD089:	CMPW	AX,#15
	BNC	$DMD890
	MOVW	AX,#15

DMD890:	MOVW	!OUTAF,AX		;Timer H1 Set
	CMPW	AX,#626
	BC	$DMD89A
	MOVW	AX,#625

DMD89A:	CMPW	AX,#15
	BNC	$DMD89B
	MOVW	AX,#15

DMD89B:	CMPW	AX,#245			;245 ~ 625
	BC	$DMD89D

	MOVW	DE,AX
	MOVW	HL,#62500
	MOVW	BC,#0FFFFH		;	BC <-- HL / DE
DMD89C:	MOV	A,L
	SUB	A,E
	MOV	L,A
	MOV	A,H
	SUBC	A,D
	MOV	H,A
	INCW	BC
	BNC	$DMD89C

	CLR1	TMHMD1.7		;	TimerH1 Stop
	MOV	TMHMD1,#00010000B	;	TimerH1 4MHz Set
	MOV	A,C			;	TimerH1 CMP01 Set
	MOV	CMP01,A
	SET1	TMHMD1.7		;	TimerH1 Start
	BR	DMD010

DMD89D:	CMPW	AX,#62			;62 ~ 244
	BC	$DMD89E

	MOV	A,X
	MOV	C,A
	MOVW	AX,#15625
	DIVUW	C

	CLR1	TMHMD1.7		;	TimerH1 Stop
	MOV	TMHMD1,#00100000B	;	TimerH1 1MHz Set
	MOV	A,X			;	TimerH1 CMP01 Set
	MOV	CMP01,A
	SET1	TMHMD1.7		;	TimerH1 Start
	BR	DMD010

DMD89E:	MOV	A,X
	MOV	C,A			;15 ~ 61
	MOVW	AX,#3750
	DIVUW	C

	CLR1	TMHMD1.7		;	TimerH1 Stop
	MOV	TMHMD1,#01110000B	;	TimerH1 240kHz Set
	MOV	A,X			;	TimerH1 CMP01 Set
	MOV	CMP01,A
	SET1	TMHMD1.7		;	TimerH1 Start
	BR	DMD010

DMD090:	CMP	A,#4			;	Long SW3 On ?
	BZ	$DMD099			;		(Yes)
	BR	DMD050

DMD099:	CLR1	TMHMD1.7		;TimerH1 Stop

	RET

;**********  TimerH1 Interupt ***********
DAINT:	PUSH	PSW			;Register Push
	PUSH	HL
	PUSH	AX
	PUSH	BC

	MOV	A,!AMODE		;[HL + B] D/A Output Data Address Set
	INC	A
	MOV	B,A

	MOVW	AX,#SINTBL-64
DAI010:	ADDW	AX,#64
	DBNZ	B,$DAI010
	MOVW	HL,AX

	MOV	A,!DACNT
	MOV	B,A
	MOV	A,[HL+B]		;A <-- D/A Data Set

	CLR1	P6.1			;***** D/A Data Output *****(CS L Set)

	MOV	CSIC10,#00011000B	;CSI Data Tx(CSI10 Clock,Edge Set)
	SET1	CSIM10.7		;CSI10 On

	MOV	SOTB10,#11101011B	;D/A chA,B Output Command Tx
DAO010:	BT	CSIM10.0,$DAO010	;Wait Tx

	MOV	SOTB10,A		;Data Tx
DAO020:	BT	CSIM10.0,$DAO020	;Wait Tx

	CLR1	CSIM10.7		;CSI10 Off
	SET1	P6.1			;CS H Set

	MOV	A,!DACNT		;D/A Output Data Address Counter + 1
	INC	A
	AND	A,#00111111B
	MOV	!DACNT,A

	POP	BC			;Register Pop
	POP	AX
	POP	HL
	POP	PSW

	RET

;*************** V Mode *****************
VMODE:	CALL	!LCDCL			;LCD ALL CLEAR

	MOV	A,#11H			;m Display
	MOV	E,#93			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,#12H			;V Display
	MOV	E,#108			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	CALL	!S3EDSP			;"SW3:Exit" Display

VMD010:	MOV	A,#10001011B		;V Read
	CALL	!ADREAD			;A/D ch0-1,Differential,Unipolar Set
	MOVW	!VIDAT,AX

	MOV	X,#44			;AX <-- VIDAT * 44
	MULU	X			;	B <-- VDAT(H) * 44
	MOV	A,X
	MOV	B,A
	MOVW	AX,!VIDAT		;	AX <-- VDAT(L) * 44
	MOV	A,#44
	MULU	X
	ADD	A,B			;	AX <-- B * H'100 + AX

	MOV	C,#9			;AX <-- AX / 9
	DIVUW	C

	CALL	!DSP40			;Data Dislplay
	CALL	!VLCD			;VRAM --> LCD

	CALL	!SWREAD			;SW Read
	MOV	A,!SWDAT
	CMP	A,#3			;	SW3 On ?
	BNZ	$VMD010			;		(No)

	RET

;*************** I Mode *****************
IMODE:	CALL	!LCDCL			;LCD ALL CLEAR

	MOV	A,#11H			;m Display
	MOV	E,#93			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,#0AH			;A Display
	MOV	E,#108			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	CALL	!S3EDSP			;"SW3:Exit" Display

IMD010:	MOV	A,#10011011B		;I Read
	CALL	!ADREAD			;A/D ch2-3,Differential,Unipolar Set
	MOVW	!VIDAT,AX

	MOV	X,#44			;AX <-- VIDAT * 44
	MULU	X			;	B <-- VDAT(H) * 44
	MOV	A,X
	MOV	B,A
	MOVW	AX,!VIDAT		;	AX <-- VDAT(L) * 44
	MOV	A,#44
	MULU	X
	ADD	A,B			;	AX <-- B * H'100 + AX

	MOV	C,#9			;AX <-- AX / 9
	DIVUW	C

	CALL	!DSP40			;Data Dislplay
	CALL	!VLCD			;VRAM --> LCD

	CALL	!SWREAD			;SW Read
	MOV	A,!SWDAT
	CMP	A,#3			;	SW3 On ?
	BNZ	$IMD010			;		(No)

	RET

;*************** R Mode *****************
RMODE:	CALL	!LCDCL			;LCD ALL CLEAR

	MOV	A,#13H			;k Display
	MOV	E,#93			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,#14H			; Display
	MOV	E,#108			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	CALL	!S3EDSP			;"SW3:Exit" Display

RMD002:	MOV	A,#10101111B		;R Read
	CALL	!ADREAD			;A/D ch4,Single End,Unipolar Sett
	MOVW	!RDAT,AX

	CMPW	AX,#03FEH		;A/D Data > H'3FE ?
	BC	$RMD005			;	(No)

	MOVW	AX,#9999
	BR	RMD020

RMD005:	MOV	X,#10			;HL <-- RDAT * 10
	MULU	X			;	B <-- VDAT(H) * 10
	MOV	A,X
	MOV	B,A
	MOVW	AX,!RDAT		;	AX <-- VDAT(L) * 10
	MOV	A,#10
	MULU	X
	ADD	A,B			;	AX <-- B * H'100 + AX
	MOVW	HL,AX			;	HL <-- AX

	MOVW	AX,!RDAT		;DE <-- RDAT
	MOVW	DE,AX

	MOV	A,#0FFH			;DE <-- (H'3FF - DE)
	SUB	A,E
	MOV	E,A
	MOV	A,#03H
	SUBC	A,D
	MOV	D,A

	MOVW	AX,DE			;DE = 0 ?
	CMPW	AX,#0
	BNZ	$RMD007			;	(Mo)
	MOVW	AX,#9999
	BR	RMD020

RMD007:	MOVW	BC,#0FFFFH		;BC <-- HL / DE
RMD010:	MOV	A,L
	SUB	A,E
	MOV	L,A
	MOV	A,H
	SUBC	A,D
	MOV	H,A
	INCW	BC
	BNC	$RMD010

	MOVW	AX,BC			;1~4 Keta Display
RMD020:	CALL	!DSP40
	CALL	!VLCD			;VRAM --> LCD

	CALL	!SWREAD			;SW Read
	MOV	A,!SWDAT
	CMP	A,#3			;	SW3 On ?
	BNZ	$RMD002			;		(No)

	RET

;*************** C Mode *****************
CMODE:	CALL	!LCDCL			;LCD ALL CLEAR

	MOV	A,#1AH			;uF Display
	MOV	E,#108			;	Column Address Set
	CALL	!LDSPC			;	Data Display
	CALL	!VLCD			;	VRAM --> LCD

	CALL	!S3EDSP			;"SW3:Exit" Display
	CALL	!VLCD			;VRAM --> LCD

CMD005:	MOVW	AX,#0			;CCNT Clear
	MOVW	!CCNT,AX

	CLR1	TMC51.7			;Timer51 Clear
	CLR1	IF1L.3			;Timer51 Interupt Flag Clear
	SET1	TMC51.7			;Timer51 Start
	SET1	P0.1			;P01 H Output

CMD010:	MOV	A,#10101111B		;C Read
	CALL	!ADREAD			;A/D ch4,Single End,Unipolar Sett

	CMPW	AX,#646			;A/D > 3.16V ?
	BNC	$CMD030			;	(Yes)

	MOVW	AX,!CCNT		;CCNT <-- CCNT + 1
	INCW	AX
	MOVW	!CCNT,AX

	CMPW	AX,#2499		;CCNT > 2499
	BNC	$CMD040			;	(Yes)

CMD020:	BF	IF1L.3,$CMD020		;40us Wait
	CLR1	IF1L.3			;	Timer51 Interupt Flag Clear
	BR	CMD010

CMD030:	CLR1	P0.1			;P01 L Output
	MOVW	AX,!CCNT		;AX <-- CCNT * 4
	MOV	X,#4			;	B <-- CCNT(H) * 4
	MULU	X
	MOV	A,X
	MOV	B,A
	MOVW	AX,!CCNT		;	AX <-- CCNT(L) * 4
	MOV	A,#4
	MULU	X
	ADD	A,B			;	AX <-- B * H'100 + AX

	CALL	!DSP13			;1~4 Keta Display
	CALL	!VLCD			;VRAM --> LCD

CMD040:	CLR1	P0.1			;P01 L Output
	MOVW	AX,#0			;CCNT Clear
	MOVW	!CCNT,AX

CMD050:	MOV	A,#10101111B		;C Read
	CALL	!ADREAD			;A/D ch4,Single End,Unipolar Sett

	CMPW	AX,#10			;A/D < 50mV ?
	BC	$CMD090			;	(Yes)

	MOVW	AX,!CCNT		;CCNT <-- CCNT + 1
	INCW	AX
	MOVW	!CCNT,AX

	CMPW	AX,#3000		;CCNT > 3000
	BC	$CMD050			;	(No)

CMD090:	CALL	!SWREAD			;SW Read
	MOV	A,!SWDAT
	CMP	A,#3			;	SW3 On ?
	BNZ	$CMD005			;		(No)

	RET

;*************** F Mode *****************
FMODE:	MOV	TMC00,#00001000B	;16Bit Timer Start(TI000 Input)

	CALL	!LCDCL			;LCD ALL CLEAR

	MOV	A,#13H			;k Display
	MOV	E,#93			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,#1BH			;Hz Display
	MOV	E,#108			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	CALL	!S3EDSP			;"SW3:Exit" Display

FMD002:	MOVW	AX,CR010		;CR010 = O ?
	CMPW	AX,#0
	BNZ	$FMD005			;	(No)
	MOVW	AX,#9999
	BR	FMD020

FMD005:	MOVW	DE,AX
	MOVW	HL,#16000
	MOVW	BC,#0FFFFH		;BC <-- HL / DE
FMD010:	MOV	A,L
	SUB	A,E
	MOV	L,A
	MOV	A,H
	SUBC	A,D
	MOV	H,A
	INCW	BC
	BNC	$FMD010

	MOVW	AX,BC			;1~4 Keta Display
FMD020:	CALL	!DSP40
	CALL	!VLCD			;VRAM --> LCD

	CALL	!SWREAD			;SW Read
	MOV	A,!SWDAT
	CMP	A,#3			;	SW3 On ?
	BNZ	$FMD002			;		(No)

	MOV	TMC00,#00000000B	;	16Bit Timer Stop

	RET

;************ PlusOut Mode **************
PLMODE:	SET1	TOC00.0			;16Bit Timer TO000 Output Enable
	MOV	TMC00,#00001100B	;16Bit Timer Start(TO000 Output)

	CALL	!LCDCL			;LCD ALL CLEAR

	MOV	A,#13H			;k Display
	MOV	E,#93			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,#1BH			;Hz Display
	MOV	E,#108			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	CALL	!PMNDSP			;Menu Display
	CALL	!PMNDSP1		;Menu Display1

	MOVW	AX,!OUTF		;CR000 Set
	BR	PMD089

PMD010:	MOVW	AX,!OUTF		;1~4 Keta Display
	CALL	!DSP40

	MOV	A,!PCPOS		;Cursol Invert
	MOV	X,#15
	MULU	X
	ADDW	AX,#VRAM+32
	MOVW	DE,AX
	CALL	!LDSPIV

	CALL	!VLCD			;VRAM --> LCD

PMD050:	CALL	!SWREAD			;SW Read
	MOV	A,!SWDAT
	CMP	A,#1			;	SW1 On ?
	BNZ	$PMD070			;		(No)

	MOV	A,!PCPOS		;PCPOS <-- PCPOS + 1
	INC	A
	AND	A,#00000011B
	MOV	!PCPOS,A
	BR	PMD010

PMD070:	CMP	A,#2			;	SW2 On ?
	BNZ	$PMD080			;		(No)

	MOV	A,!PCPOS
	CMP	A,#0
	BNZ	$PMD072
	MOVW	AX,!OUTF
	ADDW	AX,#1000
	BR	PMD078

PMD072:	CMP	A,#1
	BNZ	$PMD074
	MOVW	AX,!OUTF
	ADDW	AX,#100
	BR	PMD078

PMD074:	CMP	A,#2
	BNZ	$PMD076
	MOVW	AX,!OUTF
	ADDW	AX,#10
	BR	PMD078

PMD076:	MOVW	AX,!OUTF
	ADDW	AX,#1

PMD078:	CMPW	AX,#4001
	BC	$PMD089
	MOVW	AX,#4000
	BR	PMD089

PMD080:	CMP	A,#3			;	SW3 On ?
	BNZ	$PMD090			;		(No)

	MOV	A,!PCPOS
	CMP	A,#0
	BNZ	$PMD082
	MOVW	AX,!OUTF
	SUBW	AX,#1000
	BR	PMD088

PMD082:	CMP	A,#1
	BNZ	$PMD084
	MOVW	AX,!OUTF
	SUBW	AX,#100
	BR	PMD088

PMD084:	CMP	A,#2
	BNZ	$PMD086
	MOVW	AX,!OUTF
	SUBW	AX,#10
	BR	PMD088

PMD086:	MOVW	AX,!OUTF
	SUBW	AX,#1

PMD088:	BC	$PMD88A
	BNZ	$PMD089
PMD88A:	MOVW	AX,#1

PMD089:	MOVW	!OUTF,AX

	MOVW	DE,AX			;		CR000 <-- 8000 / OUTF - 1 Set
	MOVW	HL,#8000
	MOVW	BC,#0FFFFH		;		BC <-- HL / DE
PMD89A:	MOV	A,L
	SUB	A,E
	MOV	L,A
	MOV	A,H
	SUBC	A,D
	MOV	H,A
	INCW	BC
	BNC	$PMD89A

	MOVW	AX,BC			;		CR000 Set
	DECW	AX
	MOVW	CR000,AX
	BR	PMD010

PMD090:	CMP	A,#4			;	Long SW3 On ?
	BZ	$PMD099			;		(Yes)
	BR	PMD050

PMD099:	MOV	TMC00,#00000000B	;16Bit Timer Stop
	CLR1	TOC00.0			;16Bit Timer TO000 Output Disable
	CLR1	P0.1			;P01 L Output

	RET

;**************** SW Read ***************
SWREAD:	MOV	A,#11101111B		;SW Read1
	CALL	!ADREAD			;A/D ch5,Single End,Unipolar Set
	SUB	A,#3			;SWDAT <-- SW Data Set
	XOR	A,#0FFH
	INC	A
	MOV	!SWDAT,A

	CMP	A,#0			;SW On ?
	BZ	$SWR099			;	(No)

	MOVW	DE,#0E666H		;Wait 10ms
SWR010:	INCW	DE			;	DE <-- DE +1
	NOP
	MOVW	AX,DE
	CMPW	AX,#0FFFFH		;	DE > H'FFFF ?
	BC	$SWR010			;		(No)

	MOV	A,#11101111B		;SW Read2
	CALL	!ADREAD			;A/D ch5,Single End,Unipolar Set
	SUB	A,#3			;SWDAT <-- SW Data Set
	XOR	A,#0FFH
	INC	A

	CMP	A,!SWDAT		;SWREAD1 = SWREAD2 ?
	BZ	$SWR030			;	(Yes)
	MOV	A,#0			;SWDAT <-- 0 Set
	MOV	!SWDAT,A
	BR	SWR099

	MOVW	DE,#0
SWR030:	MOV	A,#11101111B		;SW Read
	CALL	!ADREAD			;A/D ch5,Single End,Unipolar Set
	SUB	A,#3
	XOR	A,#0FFH
	INC	A

	CMP	A,#0			;SW On ?
	BZ	$SWR099			;	(No)

	CMP	A,#3			;SW3 On ?
	BNZ	$SWR030			;	(No)

	INCW	DE			;DE <-- DE +1
	MOVW	AX,DE
	CMPW	AX,#4000H		;DE > H'4000 ?
	BC	$SWR030			;	(No)

	MOV	A,#4
	MOV	!SWDAT,A
	BR	SWR030

SWR099:	RET

;************ A/D Data Read *************
ADREAD:	DI				;Interupt Disable

	CLR1	P3.0			;CS L Set

	MOV	CSIC10,#00011010B	;CSI Data Tx(CSI10 Clock,Edge Set)
	SET1	CSIM10.7		;CSI10 On

	MOV	SOTB10,A		;Data Tx
ADR010:	BT	CSIM10.0,$ADR010	;Wait Tx

	MOV	SOTB10,#00H		;Dummy Data Tx1
ADR020:	BT	CSIM10.0,$ADR020	;Wait Tx
	MOV	A,SIO10
	MOV	X,A

	MOV	SOTB10,#00H		;Dummy Data Tx2
ADR030:	BT	CSIM10.0,$ADR030	;Wait Tx
	MOV	A,SIO10

	CLR1	CSIM10.7		;CSI10 Off
	SET1	P3.0			;CS H Set

	XCH	A,X			;AX <-- AX / 32
	MOV	C,#32
	DIVUW	C

	EI				;Interrupt Enable

	RET

;********** Main Menu Display ***********
MENDSP:	MOVW	DE,#VRAM+1		;VRAM Address Set
	MOV	C,#0
MND010:	MOVW	HL,#MENDAT
	MOV	A,[HL+C]
	SUB	A,#030H
	CALL	!LDSPC5
	INC	C
	MOV	A,C
	CMP	A,#20
	BC	$MND010

	MOVW	DE,#VRAM+123		;VRAM Address Set
	MOV	C,#0
MND020:	MOVW	HL,#MENDAT+20
	MOV	A,[HL+C]
	SUB	A,#030H
	CALL	!LDSPC5
	INC	C
	MOV	A,C
	CMP	A,#20
	BC	$MND020

	MOVW	DE,#VRAM+245		;VRAM Address Set
	MOV	C,#0
MND030:	MOVW	HL,#MENDAT+40
	MOV	A,[HL+C]
	SUB	A,#030H
	CALL	!LDSPC5
	INC	C
	MOV	A,C
	CMP	A,#20
	BC	$MND030

	MOVW	DE,#VRAM+367		;VRAM Address Set
	MOV	C,#0
MND040:	MOVW	HL,#MENDAT+60
	MOV	A,[HL+C]
	SUB	A,#030H
	CALL	!LDSPC5
	INC	C
	MOV	A,C
	CMP	A,#20
	BC	$MND040

	MOV	A,!MODE			;DE <-- Invert VRAM Address
	MOV	X,#42
	MULU	X
	INCW	AX

	CMPW	AX,#127
	BC	$MND050
	DECW	AX
	DECW	AX
	DECW	AX
	DECW	AX

	CMPW	AX,#249
	BC	$MND050
	DECW	AX
	DECW	AX
	DECW	AX
	DECW	AX

MND050:	ADDW	AX,#VRAM
	MOVW	DE,AX
	MOV	B,#35
MND060:	MOV	A,[DE]
	XOR	A,#0FFH
	MOV	[DE],A
	INCW	DE
	DBNZ	B,$MND060

	RET

MENDAT:	DB	'1:Osc@@2:Func@3:mV@@'
	DB	'4:mA@@@5:k`@@@6:uF@@'
	DB	'7:kHz@@8:Pout@@@@@@@'
	DB	'SW1:<@SW2:>@@SW3:Run'

;************ Osc Menu Display **********
OMNDSP:	MOV	A,!VRANG		;TMPADD <-- VRANG * 3 + TABLE Address
	MOV	X,#3
	MULU	X
	MOV	A,X
	MOV	B,A
	INC	B

	MOVW	AX,#VTBL-1
OMN010:	INCW	AX
	DBNZ	B,$OMN010
	MOVW	!TMPADD,AX

	MOVW	DE,#VRAM		;	VRAM Address Set
	MOV	A,#'C'-30H
	CALL	!LDSPC5
	MOV	A,#'H'-30H
	CALL	!LDSPC5
	MOV	A,#'1'-30H
	CALL	!LDSPC5
	MOV	A,#':'-30H
	CALL	!LDSPC5

	MOVW	AX,!TMPADD
	MOVW	HL,AX
	MOV	A,[HL]
	SUB	A,#030H
	CALL	!LDSPC5O

	MOVW	AX,!TMPADD
	MOVW	HL,AX
	MOV	A,[HL+1]
	SUB	A,#030H
	CALL	!LDSPC5O

	MOVW	AX,!TMPADD
	MOVW	HL,AX
	MOV	A,[HL+2]
	SUB	A,#030H
	CALL	!LDSPC5O

	MOV	A,#'V'-30H
	CALL	!LDSPC5

	MOVW	DE,#VRAM+244		;VRAM Address Set
	MOV	A,!TMODE		;	TMODE = 0 ?
	CMP	A,#0
	BNZ	$OMN020			;		(No)

	MOV	A,#'A'-30H
	CALL	!LDSPC5O
	MOV	A,#'u'-30H
	CALL	!LDSPC5O
	MOV	A,#'t'-30H
	CALL	!LDSPC5O
	MOV	A,#'o'-30H
	CALL	!LDSPC5O
	BR	OMN050

OMN020:	CMP	A,#1			;	TMODE = 1 ?
	BNZ	$OMN030			;		(No)

	MOV	A,#'N'-30H
	CALL	!LDSPC5O
	MOV	A,#'o'-30H
	CALL	!LDSPC5O
	MOV	A,#'r'-30H
	CALL	!LDSPC5O
	MOV	A,#'m'-30H
	CALL	!LDSPC5O
	BR	OMN050

OMN030:	MOV	A,#'R'-30H
	CALL	!LDSPC5O
	MOV	A,#'o'-30H
	CALL	!LDSPC5O
	MOV	A,#'l'-30H
	CALL	!LDSPC5O
	MOV	A,#'l'-30H
	CALL	!LDSPC5O

OMN050:	MOVW	DE,#VRAM+274		;VRAM Address Set
	MOV	A,!DMODE		;	DMODE = 0 ?
	CMP	A,#0
	BNZ	$OMN060			;		(No)

	MOV	A,#'R'-30H
	CALL	!LDSPC5O
	MOV	A,#'u'-30H
	CALL	!LDSPC5O
	MOV	A,#'n'-30H
	CALL	!LDSPC5
	BR	OMN070

OMN060:	MOV	A,#'S'-30H
	CALL	!LDSPC5O
	MOV	A,#'t'-30H
	CALL	!LDSPC5O
	MOV	A,#'p'-30H
	CALL	!LDSPC5

OMN070:	MOV	A,!TRANG		;TMPADD <-- TRANG * 4 + TABLE Address
	MOV	X,#4
	MULU	X
	MOV	A,X
	MOV	B,A
	INC	B

	MOVW	AX,#TTBL-1
OMN080:	INCW	AX
	DBNZ	B,$OMN080
	MOVW	!TMPADD,AX

	MOVW	DE,#VRAM+366		;	VRAM Address Set
	MOVW	AX,!TMPADD
	MOVW	HL,AX
	MOV	A,[HL]
	SUB	A,#030H
	CALL	!LDSPC5O

	MOVW	AX,!TMPADD
	MOVW	HL,AX
	MOV	A,[HL+1]
	SUB	A,#030H
	CALL	!LDSPC5O

	MOVW	AX,!TMPADD
	MOVW	HL,AX
	MOV	A,[HL+2]
	SUB	A,#030H
	CALL	!LDSPC5O

	MOVW	AX,!TMPADD
	MOVW	HL,AX
	MOV	A,[HL+3]
	SUB	A,#030H
	CALL	!LDSPC5O

	MOV	A,#'s'-30H
	CALL	!LDSPC5

	MOVW	DE,#VRAM+396		;VRAM Address Set
	MOV	A,#'C'-30H
	CALL	!LDSPC5
	MOV	A,#'1'-30H
	CALL	!LDSPC5
	MOV	A,#'^'-30H
	CALL	!LDSPC5

	RET

VTBL:	DB	'0;10;20;51;02;05;0'
TTBL:	DB	'0;5m1;0m2;0m5;0m@10m@20m@50m100m@0;2@0;5@1;0'

;********* D/A Out Menu Display *********
AMDSP:	MOV	A,!AMODE
	CMP	A,#0
	BNZ	$AMD010

	MOVW	DE,#VRAM		;VRAM Address Set
	MOV	A,#'S'-30H
	CALL	!LDSPC5A
	MOV	A,#'i'-30H
	CALL	!LDSPC5A
	MOV	A,#'n'-30H
	CALL	!LDSPC5A
	MOV	A,#'@'-30H
	CALL	!LDSPC5A
	BR	AMD099

AMD010:	CMP	A,#1
	BNZ	$AMD020

	MOVW	DE,#VRAM		;VRAM Address Set
	MOV	A,#'['-30H
	CALL	!LDSPC5A
	MOV	A,#'\'-30H
	CALL	!LDSPC5A
	MOV	A,#'['-30H
	CALL	!LDSPC5A
	MOV	A,#'\'-30H
	CALL	!LDSPC5A
	BR	AMD099

AMD020:	MOVW	DE,#VRAM		;VRAM Address Set
	MOV	A,#']'-30H
	CALL	!LDSPC5A
	MOV	A,#']'-30H
	CALL	!LDSPC5A
	MOV	A,#']'-30H
	CALL	!LDSPC5A
	MOV	A,#']'-30H
	CALL	!LDSPC5A

AMD099:	RET

;******* Pulse Out Menu Display 1 *******
PMNDSP1:MOVW	DE,#VRAM		;VRAM Address Set
	MOV	A,#'='-30H
	CALL	!LDSPC5A
	MOV	A,#'?'-30H
	CALL	!LDSPC5A
	MOV	A,#'='-30H
	CALL	!LDSPC5A
	MOV	A,#'?'-30H
	CALL	!LDSPC5A
	MOV	A,#'='-30H
	CALL	!LDSPC5A

	RET

;******** Pulse Out Menu Display ********
PMNDSP:	MOVW	DE,#VRAM+122		;VRAM Address Set
	MOV	A,#'S'-30H
	CALL	!LDSPC5
	MOV	A,#'W'-30H
	CALL	!LDSPC5
	MOV	A,#'1'-30H
	CALL	!LDSPC5
	MOV	A,#':'-30H
	CALL	!LDSPC5
	MOV	A,#'>'-30H
	CALL	!LDSPC5

	MOVW	DE,#VRAM+244		;VRAM Address Set
	MOV	A,#'S'-30H
	CALL	!LDSPC5
	MOV	A,#'W'-30H
	CALL	!LDSPC5
	MOV	A,#'2'-30H
	CALL	!LDSPC5
	MOV	A,#':'-30H
	CALL	!LDSPC5
	MOV	A,#'^'-30H
	CALL	!LDSPC5

	MOVW	DE,#VRAM+366		;VRAM Address Set
	MOV	A,#'S'-30H
	CALL	!LDSPC5
	MOV	A,#'W'-30H
	CALL	!LDSPC5
	MOV	A,#'3'-30H
	CALL	!LDSPC5
	MOV	A,#':'-30H
	CALL	!LDSPC5
	MOV	A,#'_'-30H
	CALL	!LDSPC5
	MOV	A,#'@'-30H
	CALL	!LDSPC5
	MOV	A,#'@'-30H
	CALL	!LDSPC5
	MOV	A,#'L'-30H
	CALL	!LDSPC5
	MOV	A,#'o'-30H
	CALL	!LDSPC5
	MOV	A,#'n'-30H
	CALL	!LDSPC5
	MOV	A,#'g'-30H
	CALL	!LDSPC5
	MOV	A,#'@'-30H
	CALL	!LDSPC5
	MOV	A,#'S'-30H
	CALL	!LDSPC5
	MOV	A,#'W'-30H
	CALL	!LDSPC5
	MOV	A,#'3'-30H
	CALL	!LDSPC5
	MOV	A,#':'-30H
	CALL	!LDSPC5
	MOV	A,#'E'-30H
	CALL	!LDSPC5
	MOV	A,#'x'-30H
	CALL	!LDSPC5
	MOV	A,#'i'-30H
	CALL	!LDSPC5
	MOV	A,#'t'-30H
	CALL	!LDSPC5

	RET

;*********** "SW3:Exit"Display **********
S3EDSP:	MOVW	DE,#VRAM+439		;VRAM Address Set

	MOV	A,#'S'-30H
	CALL	!LDSPC5
	MOV	A,#'W'-30H
	CALL	!LDSPC5
	MOV	A,#'3'-30H
	CALL	!LDSPC5
	MOV	A,#':'-30H
	CALL	!LDSPC5
	MOV	A,#'E'-30H
	CALL	!LDSPC5
	MOV	A,#'x'-30H
	CALL	!LDSPC5
	MOV	A,#'i'-30H
	CALL	!LDSPC5
	MOV	A,#'t'-30H
	CALL	!LDSPC5

	RET

;************* Waku Display *************
WAKDSP:	MOVW	HL,#VRAM+49+122*0
	MOV	X,#056H
	MOV	D,#002H
	MOV	E,#012H
	CALL	!WDSP

	MOVW	HL,#VRAM+49+122*1
	MOV	X,#055H
	MOV	D,#000H
	MOV	E,#004H
	CALL	!WDSP

	MOVW	HL,#VRAM+49+122*2
	MOV	X,#055H
	MOV	D,#000H
	MOV	E,#041H
	CALL	!WDSP

	MOVW	HL,#VRAM+49+122*3
	MOV	X,#0D5H
	MOV	D,#080H
	MOV	E,#090H
	CALL	!WDSP

	MOV	A,#0FFH
	MOV	!(VRAM+49+122*1),A
	MOV	!(VRAM+49+122*2),A
	MOV	!(VRAM+49+122*3),A
	MOV	!(VRAM+121+122*1),A
	MOV	!(VRAM+121+122*2),A
	MOV	!(VRAM+121+122*3),A

	MOV	A,#0FEH
	MOV	!(VRAM+49+122*0),A
	MOV	!(VRAM+121+122*0),A

	MOV	A,#0FFH			;Trig Time Position Set
	MOV	!(VRAM+49),A

	RET

WDSP:	MOV	B,#12
WDSP10:	MOV	A,X
	MOV	[HL],A
	INCW	HL

	MOV	A,D
	MOV	[HL],A
	INCW	HL

	MOV	A,E
	MOV	[HL],A
	INCW	HL

	MOV	A,D
	MOV	[HL],A
	INCW	HL

	MOV	A,E
	MOV	[HL],A
	INCW	HL

	MOV	A,D
	MOV	[HL],A
	INCW	HL

	DBNZ	B,$WDSP10

	RET

;********* 1~3Keta Display(00.0) ********
DSP21:	MOV	B,#0FFH			;DSPD1~4 Set
DSP210:	INC	B
	SUB	A,#100
	BNC	$DSP210
	ADD	A,#100

	MOV	C,#0FFH
DSP220:	INC	C
	SUB	A,#10
	BNC	$DSP220
	ADD	A,#10

	MOV	D,A

	MOV	A,B
	MOV	!DSPD1,A
	MOV	A,C
	MOV	!DSPD2,A
	MOV	A,D
	MOV	!DSPD3,A

	MOV	A,#0			;DSPD1 Display
	CMP	A,!DSPD1		;	DSPD1 = 0 ?
	BNZ	$DSP250			;		(No)
	MOV	A,#10H
	MOV	!DSPD1,A
DSP250:	MOV	A,!DSPD1
	MOV	E,#32			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,!DSPD2		;DSPD2 Display
	MOV	E,#47			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,#19H			;'.'  Display
	MOV	E,#62			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,!DSPD3		;DSPD3 Display
	MOV	E,#77			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	RET

;********* 1~4Keta Display(0000) ********
DSP40:	MOV	B,#0FFH			;DSPD1~4 Set
DSP410:	INC	B
	SUBW	AX,#1000
	BNC	$DSP410
	ADDW	AX,#1000

	MOV	C,#0FFH
DSP420:	INC	C
	SUBW	AX,#100
	BNC	$DSP420
	ADDW	AX,#100

	MOV	D,#0FFH
DSP430:	INC	D
	SUBW	AX,#10
	BNC	$DSP430
	ADDW	AX,#10

	MOV	A,B
	MOV	!DSPD1,A
	MOV	A,C
	MOV	!DSPD2,A
	MOV	A,D
	MOV	!DSPD3,A
	MOV	A,X
	MOV	!DSPD4,A

	MOV	A,#0			;DSPD1 Display
	CMP	A,!DSPD1		;	DSPD1 = 0 ?
	BNZ	$DSP450			;		(No)
	MOV	A,#10H
	MOV	!DSPD1,A
DSP450:	MOV	A,!DSPD1
	MOV	E,#32			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,#10H			;DSPD2 Display
	CMP	A,!DSPD1		;	DSPD1 = 0 ?
	BNZ	$DSP460			;		(No)
	MOV	A,#0			;	DSPD2 = 0 ?
	CMP	A,!DSPD2
	BNZ	$DSP460			;		(No)
	MOV	A,#10H
	MOV	!DSPD2,A
DSP460:	MOV	A,!DSPD2
	MOV	E,#47			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,#10H			;DSPD3 Display
	CMP	A,!DSPD1		;	DSPD1 = 0 ?
	BNZ	$DSP470			;		(No)
	CMP	A,!DSPD2		;	DSPD2 = 0 ?
	BNZ	$DSP470			;		(No)
	MOV	A,#0			;	DSPD3 = 0 ?
	CMP	A,!DSPD3
	BNZ	$DSP470			;		(No)
	MOV	A,#10H
	MOV	!DSPD3,A
DSP470:	MOV	A,!DSPD3
	MOV	E,#62			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,!DSPD4		;DSPD4 Display
	MOV	E,#77			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	RET

;******** 1~4Keta Display(0.000) ********
DSP13:	MOV	B,#0FFH			;DSPD1~4 Set
DSP131:	INC	B
	SUBW	AX,#1000
	BNC	$DSP131
	ADDW	AX,#1000

	MOV	C,#0FFH
DSP132:	INC	C
	SUBW	AX,#100
	BNC	$DSP132
	ADDW	AX,#100

	MOV	D,#0FFH
DSP133:	INC	D
	SUBW	AX,#10
	BNC	$DSP133
	ADDW	AX,#10

	MOV	A,B
	MOV	!DSPD1,A
	MOV	A,C
	MOV	!DSPD2,A
	MOV	A,D
	MOV	!DSPD3,A
	MOV	A,X
	MOV	!DSPD4,A

	MOV	A,!DSPD1		;DSPD1 Display
	MOV	E,#32			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,#19H			;'.'  Display
	MOV	E,#47			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,!DSPD2		;DSPD2 Display
	MOV	E,#62			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,!DSPD3		;DSPD3 Display
	MOV	E,#77			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOV	A,!DSPD4		;DSPD4 Display
	MOV	E,#92			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	RET

;***** DSPDAT Display(H'0000) ***********
CHRDSP:	MOVW	AX,!DSPDAT		;A <-- Display Data Set
	ROR	A,1
	ROR	A,1
	ROR	A,1
	ROR	A,1
	AND	A,#0FH
	MOV	E,#32			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOVW	AX,!DSPDAT		;A <-- Display Data Set
	AND	A,#0FH
	MOV	E,#47			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOVW	AX,!DSPDAT		;A <-- Display Data Set
	MOV	A,X
	ROR	A,1
	ROR	A,1
	ROR	A,1
	ROR	A,1
	AND	A,#0FH
	MOV	E,#63			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	MOVW	AX,!DSPDAT		;A <-- Display Data Set
	MOV	A,X
	AND	A,#0FH
	MOV	E,#78			;	Column Address Set
	CALL	!LDSPC			;	Data Display

	RET

;********** Osc Trig Level Set **********
OTPDSP:	MOV	A,!TLVL
	MOV	B,A
	INC	B

	MOV	A,#001H
	MOV	!(VRAM+48),A
	MOV	A,#000H
	MOV	!(VRAM+122+48),A
	MOV	!(VRAM+244+48),A
	MOV	!(VRAM+366+48),A

	CLR1	CY
OTP010:	MOV	A,!(VRAM+48)
	ROLC	A,1
	MOV	!(VRAM+48),A
	MOV	A,!(VRAM+122+48)
	ROLC	A,1
	MOV	!(VRAM+122+48),A
	MOV	A,!(VRAM+244+48)
	ROLC	A,1
	MOV	!(VRAM+244+48),A
	MOV	A,!(VRAM+366+48)
	ROLC	A,1
	MOV	!(VRAM+366+48),A
	DBNZ	B,$OTP010

	MOV	A,!TLVL
	MOV	B,A
	INC	B

	MOV	A,#003H
	MOV	!(VRAM+47),A
	MOV	A,#000H
	MOV	!(VRAM+122+47),A
	MOV	!(VRAM+244+47),A
	MOV	!(VRAM+366+47),A

	SET1	CY
OTP020:	MOV	A,!(VRAM+47)
	ROLC	A,1
	MOV	!(VRAM+47),A

	MOV	A,!(VRAM+122+47)
	ROLC	A,1
	MOV	!(VRAM+122+47),A

	MOV	A,!(VRAM+244+47)
	ROLC	A,1
	MOV	!(VRAM+244+47),A

	MOV	A,!(VRAM+366+47)
	ROLC	A,1
	MOV	!(VRAM+366+47),A

	DBNZ	B,$OTP020

	RET

;*********** Osc Position Set ***********
OPOSDP:	MOV	A,!(VRAM+244+48)
	OR	A,#001H
	MOV	!(VRAM+244+48),A

	RET

;********* Osc Mode Menu Invert *********
OMENIV:	MOV	A,!OCPOS
	CMP	A,#0			;OCPOS = 0 ?
	BNZ	$OIV020
	MOV	B,#23
	MOVW	DE,#VRAM+24
OIV010:	MOV	A,[DE]
	XOR	A,#0FFH
	MOV	[DE],A
	INCW	DE
	DBNZ	B,$OIV010
	BR	OIV099

OIV020:	CMP	A,#1			;OCPOS = 1 ?
	BNZ	$OIV040
	MOV	B,#23
	MOVW	DE,#VRAM+244
OIV030:	MOV	A,[DE]
	XOR	A,#0FFH
	MOV	[DE],A
	INCW	DE
	DBNZ	B,$OIV030
	BR	OIV099

OIV040:	CMP	A,#2			;OCPOS = 2 ?
	BNZ	$OIV060
	MOV	B,#17
	MOVW	DE,#VRAM+244+30
OIV050:	MOV	A,[DE]
	XOR	A,#0FFH
	MOV	[DE],A
	INCW	DE
	DBNZ	B,$OIV050
	BR	OIV099

OIV060:	CMP	A,#3			;OCPOS = 3 ?
	BNZ	$OIV080
	MOV	B,#29
	MOVW	DE,#VRAM+366
OIV070:	MOV	A,[DE]
	XOR	A,#0FFH
	MOV	[DE],A
	INCW	DE
	DBNZ	B,$OIV070
	BR	OIV099

OIV080:	MOV	A,!VRAM+47		;OCPOS = 4
	XOR	A,#0FFH
	MOV	!VRAM+47,A

	MOV	A,!VRAM+122+47
	XOR	A,#0FFH
	MOV	!VRAM+122+47,A

	MOV	A,!VRAM+244+47
	XOR	A,#0FFH
	MOV	!VRAM+244+47,A

	MOV	A,!VRAM+366+47
	XOR	A,#0FFH
	MOV	!VRAM+366+47,A

OIV099:	RET

;********* D/A Mode Menu Invert *********
AMENIV:	MOV	A,!ACPOS
	CMP	A,#0
	BNZ	$AIV020
	MOV	B,#20
	MOVW	DE,#VRAM
AIV010:	MOV	A,[DE]
	XOR	A,#0FFH
	MOV	[DE],A
	INCW	DE
	DBNZ	B,$AIV010
	BR	AIV099

AIV020:	CMP	A,#1
	BNZ	$AIV030
	MOVW	DE,#VRAM+47
	CALL	!LDSPIV
	BR	AIV099

AIV030:	CMP	A,#2
	BNZ	$AIV040
	MOVW	DE,#VRAM+62
	CALL	!LDSPIV
	BR	AIV099

AIV040:	MOVW	DE,#VRAM+77
	CALL	!LDSPIV

AIV099:	RET

;**** LCD Character Invert (14*24) *****
;   (DE):VRAM Address
LDSPIV:	MOV	C,#3			;Page Counter 4 Set
LDI010:	MOV	B,#14			;Column Counter 14 Set
LDI020:	MOV	A,[DE]
	XOR	A,#0FFH
	MOV	[DE],A
	INCW	DE
	DBNZ	B,$LDI020

	MOVW	AX,DE			;DE <-- DE + 122 - 14
	ADDW	AX,#122-14
	MOVW	DE,AX

	DBNZ	C,$LDI010

	RET

;**** LCD Character Display (14*24) *****
;   (A):Number   (E):Column Address
LDSPC:	MOV	X,#42			;HL <-- Display Font Address Set
	MULU	X
	ADDW	AX,#FONT
	MOVW	HL,AX

	MOV	D,#0			;DE <-- VRAM Address Set
	MOVW	AX,DE
	ADDW	AX,#VRAM
	MOVW	DE,AX

	MOV	C,#3			;Page Counter 4 Set
LDC010:	MOV	B,#14			;Column Counter 14 Set
LDC020:	MOV	A,[HL]
	MOV	[DE],A
	INCW	HL
	INCW	DE
	DBNZ	B,$LDC020

	MOVW	AX,DE			;DE <-- DE + 122 - 14
	ADDW	AX,#122-14
	MOVW	DE,AX

	DBNZ	C,$LDC010

	RET

;***** LCD Character Display (5*7) *****
;   (A):Number   (DE):VRAM Address
LDSPC5O:MOV	X,#5			;HL <-- Display Font Address Set
	MULU	X
	ADDW	AX,#FONT5
	MOVW	HL,AX

	MOV	B,#5			;Column Counter 5 Set
LDC520B:MOV	A,[HL]
	MOV	[DE],A
	INCW	HL
	INCW	DE
	DBNZ	B,$LDC520B
	MOV	A,#0
	MOV	[DE],A
	INCW	DE

	RET

;***** LCD Character Display (5*7) *****
;   (A):Number   (DE):VRAM Address
LDSPC5A:MOV	X,#5			;HL <-- Display Font Address Set
	MULU	X
	ADDW	AX,#FONT5
	MOVW	HL,AX

	MOV	B,#5			;Column Counter 5 Set
LDC520A:MOV	A,[HL]
	MOV	[DE],A
	INCW	HL
	INCW	DE
	DBNZ	B,$LDC520A

	RET

;***** LCD Character Display (5*7) *****
;   (A):Number   (DE):VRAM Address
LDSPC5:	MOV	X,#5			;HL <-- Display Font Address Set
	MULU	X
	ADDW	AX,#FONT5
	MOVW	HL,AX

	MOV	B,#5			;Column Counter 5 Set
LDC520:	MOV	A,[HL]
	MOV	[DE],A
	INCW	HL
	INCW	DE
	DBNZ	B,$LDC520
	INCW	DE

	RET

;************ LCD ALL CLEAR *************
LCDCL:	MOVW	HL,#VRAM		;HL <-- VRAM Address Set
	MOV	A,#0
	MOV	C,#4
LCC010:	MOV	B,#122

LCC020:	MOV	[HL],A
	INCW	HL
	DBNZ	B,$LCC020
	DBNZ	C,$LCC010

	RET

;********** VRAM --> LCD Output *********
VLCD:	MOVW	HL,#VRAM		;HL <-- VRAM Address Set(CS1)
	CLR1	P3.3			;	CS1 L Set
	CALL	!VLCO			;	VRAM --> LCD
	SET1	P3.3			;	CS1 H Set

	MOVW	HL,#VRAM+61		;HL <-- VRAM Address Set(CS2)
	CLR1	P1.5			;	CS2 L Set
	CALL	!VLCO			;	VRAM --> LCD
	SET1	P1.5			;	CS2 H Set

	RET

;**** VRAM --> LCD (HL:VRAM Address) ****
VLCO:	MOV	D,#0B8H			;D <-- Page Address 0 Set
	MOV	C,#4			;Page Counter 4 Set
VLCO40:	CLR1	P1.6			;Page Loop (Data/Instruction I Set)
	MOV	A,D			;	Page Address Set
	CALL	!SROUT
	MOV	A,#0			;	Column Address  0 Set
	CALL	!SROUT
	MOV	A,#0AFH			;	Display On
	CALL	!SROUT

	SET1	P1.6			;	Data/Instruction D Set
	MOV	B,#61			;	Column Counter 61 Set
VLCO50:	MOV	A,[HL]			;		VRAM Data Output Loop
	CALL	!SROUT
	INCW	HL
	DBNZ	B,$VLCO50		;		Loop End

	MOVW	AX,HL			;	HL(VRAM Address) <-- HL + 61
	ADDW	AX,#61
	MOVW	HL,AX
	INC	D			;	Page Address(D) +1
	DBNZ	C,$VLCO40		;Loop End

	RET

;******** Shift Register Output *********
SROUT:	DI				;Interupt Disable
	SET1	P6.0			;E H Set
	MOV	CSIC10,#00011001B	;CSI Data Tx(CSI10 Clock,Edge Set)
	SET1	CSIM10.7		;	CSI10 On
	MOV	SOTB10,A		;	Data Tx
SRO010:	BT	CSIM10.0,$SRO010	;	Wait Tx
	CLR1	CSIM10.7		;	CSI10 Off

	SET1	P12.0			;Strobe H Set
	CLR1	P12.0			;Strobe L Set

	CLR1	P6.0			;E L Set
	CLR1	P6.0			;E L Set
	EI				;Interrupt Enable

	RET

;ROMF	CSEG	AT	02000H		;ROM
;****************************************
;*	      Fonts Table		*
;****************************************
FONT:	DB	080H,0E0H,0F8H,0FCH,07EH,01FH,00FH,00FH,01FH,07EH,0FCH,0F8H,0E0H,080H	;0
	DB	0FFH,0FFH,0FFH,0FFH,000H,000H,000H,000H,000H,000H,0FFH,0FFH,0FFH,0FFH
	DB	001H,007H,01FH,03FH,07EH,0F8H,0F0H,0F0H,0F8H,07EH,03FH,01FH,007H,001H
	DB	000H,000H,0F0H,0F0H,0F8H,0F8H,0FCH,0FEH,0FFH,0FFH,0FFH,0FFH,000H,000H	;1
	DB	000H,000H,000H,000H,000H,000H,000H,0FFH,0FFH,0FFH,0FFH,0FFH,000H,000H
	DB	000H,000H,000H,000H,000H,000H,000H,0FFH,0FFH,0FFH,0FFH,0FFH,000H,000H
	DB	000H,0F0H,0F8H,0FCH,0FEH,03FH,01FH,00FH,00FH,01EH,0FEH,0FCH,0F8H,0C0H	;2
	DB	000H,001H,001H,001H,081H,0C0H,0E0H,0F0H,0F8H,07CH,03FH,01FH,00FH,003H
	DB	0F0H,0F8H,0FEH,0FFH,0FFH,0F7H,0F3H,0F1H,0F0H,0F0H,0F0H,0F0H,0F0H,0F0H
	DB	0E0H,0F0H,0FCH,0FEH,03EH,00FH,00FH,00FH,03FH,0FEH,0FEH,0FCH,0F0H,000H	;3
	DB	080H,080H,080H,080H,000H,008H,01CH,03EH,03EH,0FFH,0FFH,0F7H,0E3H,0C0H
	DB	003H,01FH,03FH,07FH,07CH,078H,0F0H,0F0H,078H,07FH,07FH,03FH,01FH,00FH
	DB	000H,000H,000H,080H,0E0H,0F8H,0FEH,07FH,0FFH,0FFH,0FFH,0FFH,000H,000H	;4
	DB	0E0H,0F8H,0FEH,0FFH,09FH,087H,081H,080H,0FFH,0FFH,0FFH,0FFH,080H,080H
	DB	007H,007H,007H,007H,007H,007H,007H,007H,0FFH,0FFH,0FFH,0FFH,007H,007H
	DB	000H,0F0H,0FFH,0FFH,0FFH,00FH,00FH,00FH,00FH,00FH,00FH,00FH,00FH,000H	;5
	DB	010H,01FH,01FH,01FH,00FH,007H,007H,007H,00FH,01EH,0FEH,0FEH,0FCH,0E0H
	DB	003H,01FH,03FH,07FH,078H,0F0H,0F0H,0F0H,0F8H,07CH,07FH,03FH,00FH,001H
	DB	080H,0F0H,0FCH,0FEH,03EH,01FH,00FH,00FH,00FH,01FH,07EH,07CH,078H,060H	;6
	DB	0FFH,0FFH,0FFH,0FFH,03CH,01EH,00FH,00FH,00FH,01FH,0FEH,0FCH,0F8H,0E0H
	DB	001H,00FH,01FH,03FH,07CH,078H,0F0H,0F0H,0F0H,07CH,07FH,03FH,01FH,003H
	DB	00FH,00FH,00FH,00FH,00FH,00FH,00FH,00FH,0CFH,0EFH,0FFH,0FFH,03FH,01FH	;7
	DB	000H,000H,000H,000H,080H,0F0H,0FCH,0FFH,07FH,00FH,003H,000H,000H,000H
	DB	000H,000H,000H,0F0H,0FFH,0FFH,0FFH,00FH,000H,000H,000H,000H,000H,000H
	DB	000H,0F8H,0FCH,0FEH,0FFH,00FH,007H,007H,00FH,0FFH,0FEH,0FCH,0F8H,000H	;8
	DB	0C0H,0F1H,0FBH,0FFH,07FH,03EH,01CH,01CH,03EH,07FH,0FFH,0FBH,0F1H,0C0H
	DB	007H,01FH,03FH,07FH,078H,0F0H,0E0H,0E0H,0F0H,078H,07FH,03FH,01FH,007H
	DB	0C0H,0F0H,0FCH,0FEH,03EH,01FH,00FH,00FH,01FH,03EH,0FEH,0FCH,0F0H,0C0H	;9
	DB	00FH,03FH,07FH,07FH,0F8H,0F0H,0E0H,0E0H,0F0H,078H,0FFH,0FFH,0FFH,0FFH
	DB	000H,00CH,03CH,07CH,07CH,0F1H,0F1H,0F1H,0F8H,07EH,07FH,03FH,00FH,001H
	DB	000H,000H,000H,000H,0E0H,0F8H,0FFH,0FFH,0F8H,0E0H,000H,000H,000H,000H	;A
	DB	000H,000H,0E0H,0FFH,0FFH,09FH,080H,080H,09FH,0FFH,0FFH,0E0H,000H,000H
	DB	0E0H,0FEH,0FFH,03FH,01FH,007H,007H,007H,007H,01FH,03FH,0FFH,0FEH,0E0H
	DB	0FFH,0FFH,0FFH,0FFH,00FH,00FH,00FH,00FH,00EH,01EH,0FEH,0FCH,0F8H,000H	;B
	DB	0FFH,0FFH,0FFH,0FFH,03CH,03CH,03CH,03CH,03EH,07FH,0F7H,0F7H,0E3H,080H
	DB	0FFH,0FFH,0FFH,0FFH,0F0H,0F0H,0F0H,0F0H,0F0H,078H,07FH,07FH,03FH,00FH
	DB	000H,0F0H,0F8H,0FCH,03EH,01EH,01FH,01FH,01FH,03FH,0FEH,0FCH,0F8H,0E0H	;C
	DB	0FFH,0FFH,0FFH,0FFH,000H,000H,000H,000H,000H,000H,001H,001H,001H,001H
	DB	000H,00FH,01FH,03FH,078H,070H,0F0H,0F0H,0F0H,0F8H,07FH,03FH,01FH,007H
	DB	0FFH,0FFH,0FFH,0FFH,00FH,00FH,00FH,00FH,01EH,07EH,0FCH,0F8H,0E0H,080H	;D
	DB	0FFH,0FFH,0FFH,0FFH,000H,000H,000H,000H,000H,000H,0FFH,0FFH,0FFH,0FFH
	DB	0FFH,0FFH,0FFH,0FFH,0F0H,0F0H,0F0H,0F0H,078H,07EH,03FH,01FH,007H,001H
	DB	0FFH,0FFH,0FFH,0FFH,00FH,00FH,00FH,00FH,00FH,00FH,00FH,00FH,00FH,00FH	;E
	DB	0FFH,0FFH,0FFH,0FFH,03CH,03CH,03CH,03CH,03CH,03CH,03CH,03CH,03CH,000H
	DB	0FFH,0FFH,0FFH,0FFH,0F0H,0F0H,0F0H,0F0H,0F0H,0F0H,0F0H,0F0H,0F0H,0F0H
	DB	0FFH,0FFH,0FFH,0FFH,00FH,00FH,00FH,00FH,00FH,00FH,00FH,00FH,00FH,00FH	;F
	DB	0FFH,0FFH,0FFH,0FFH,03CH,03CH,03CH,03CH,03CH,03CH,03CH,03CH,03CH,000H
	DB	0FFH,0FFH,0FFH,0FFH,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H
	DB	000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H	;" "
	DB	000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H
	DB	000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H
	DB	000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H	;m
	DB	0F0H,0F0H,0F0H,060H,070H,0F0H,0E0H,0C0H,0E0H,070H,070H,0F0H,0E0H,0C0H
	DB	0FFH,0FFH,0FFH,000H,000H,0FFH,0FFH,0FFH,000H,000H,000H,0FFH,0FFH,0FFH
	DB	003H,0FFH,0FFH,0FFH,0C0H,000H,000H,000H,000H,0C0H,0FFH,0FFH,07FH,003H	;V
	DB	000H,000H,007H,0FFH,0FFH,0F8H,080H,080H,0F8H,0FFH,0FFH,007H,000H,000H
	DB	000H,000H,000H,000H,01FH,0FFH,0FFH,0FFH,0FFH,01FH,000H,000H,000H,000H
	DB	0FFH,0FFH,0FFH,0FFH,000H,000H,000H,080H,0C0H,0C0H,0C0H,0C0H,040H,000H	;k
	DB	0FFH,0FFH,0FFH,0FFH,0F0H,0FCH,0FFH,0FFH,0EFH,087H,003H,000H,000H,000H
	DB	0FFH,0FFH,0FFH,0FFH,001H,000H,007H,01FH,07FH,0FFH,0FEH,0F8H,0E0H,080H
	DB	080H,0F0H,0F8H,0FEH,03EH,01FH,00FH,00FH,01FH,03EH,0FEH,0F8H,0F0H,0C0H	;
	DB	03FH,0FFH,0FFH,0FFH,080H,000H,000H,000H,000H,080H,0FFH,0FFH,0FFH,03FH
	DB	000H,0F1H,0F3H,0F7H,0FFH,0FEH,000H,000H,0FEH,0FFH,0F7H,0F3H,0F1H,000H
	DB	000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H	;u
	DB	0FFH,0FFH,0FFH,000H,000H,000H,000H,000H,000H,000H,000H,0FFH,0FFH,0FFH
	DB	0FFH,0FFH,0FFH,006H,00CH,00CH,00CH,00CH,00CH,00EH,007H,00FH,00FH,00FH
	DB	000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H	;DT
	DB	000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H
	DB	000H,000H,000H,000H,030H,078H,0FCH,0FCH,078H,030H,000H,000H,000H,000H
	DB	0FFH,0FFH,0FFH,0FFH,000H,000H,000H,000H,000H,000H,0FFH,0FFH,0FFH,0FFH	;H
	DB	0FFH,0FFH,0FFH,0FFH,03CH,03CH,03CH,03CH,03CH,03CH,0FFH,0FFH,0FFH,0FFH
	DB	0FFH,0FFH,0FFH,0FFH,000H,000H,000H,000H,000H,000H,0FFH,0FFH,0FFH,0FFH
	DB	000H,080H,080H,080H,080H,080H,080H,080H,080H,080H,080H,080H,080H,000H	;z
	DB	000H,007H,007H,007H,087H,0C7H,0E7H,0F7H,0FFH,07FH,03FH,01FH,00FH,000H
	DB	0F0H,0F8H,0FCH,0FFH,0FFH,0FFH,0F7H,0F3H,0F0H,0F0H,0F0H,0F0H,0F0H,0F0H
	DB	000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H	;Dot
	DB	000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H,000H
	DB	000H,000H,000H,000H,030H,078H,0FCH,0FCH,078H,030H,000H,000H,000H,000H
	DB	0C0H,0C0H,000H,000H,0C0H,0C0H,000H,0FFH,0FFH,00FH,00FH,00FH,00FH,00FH	;uF
	DB	0FFH,0FFH,000H,000H,0FFH,0FFH,000H,0FFH,0FFH,01CH,01CH,01CH,01CH,000H
	DB	03FH,0FFH,0F0H,070H,0FFH,0FFH,000H,0FFH,0FFH,000H,000H,000H,000H,000H
	DB	0FFH,0FFH,000H,000H,000H,0FFH,0FFH,000H,000H,0C0H,0C0H,0C0H,0C0H,0C0H	;Hz
	DB	0FFH,0FFH,03CH,03CH,03CH,0FFH,0FFH,000H,000H,003H,083H,0FBH,07FH,007H
	DB	0FFH,0FFH,000H,000H,000H,0FFH,0FFH,000H,000H,0FEH,0FFH,0F1H,0F0H,0F0H

FONT5:	DB	07CH,0A2H,092H,08AH,07CH,000H,084H,0FEH,080H,000H,084H,0C2H,0A2H,092H,08CH,042H,082H,08AH,096H,062H
	DB	030H,028H,024H,0FEH,020H,04EH,08AH,08AH,08AH,072H,078H,094H,092H,092H,060H,002H,0E2H,012H,00AH,006H
	DB	06CH,092H,092H,092H,06CH,00CH,092H,092H,052H,03CH,000H,06CH,06CH,000H,000H,000H,000H,0C0H,0C0H,000H
	DB	010H,038H,054H,010H,010H,080H,080H,080H,080H,0FEH,010H,010H,054H,038H,010H,002H,002H,002H,002H,0FEH
	DB	000H,000H,000H,000H,000H,0FCH,022H,022H,022H,0FCH,0FEH,092H,092H,092H,06CH,07CH,082H,082H,082H,044H
	DB	0FEH,082H,082H,044H,038H,0FEH,092H,092H,092H,082H,0FEH,012H,012H,012H,002H,07CH,082H,092H,092H,0F4H
	DB	0FEH,010H,010H,010H,0FEH,000H,082H,0FEH,082H,000H,040H,080H,082H,07EH,002H,0FEH,010H,028H,044H,082H
	DB	0FEH,080H,080H,080H,080H,0FEH,004H,018H,004H,0FEH,0FEH,008H,010H,020H,0FEH,07CH,082H,082H,082H,07CH
	DB	0FEH,012H,012H,012H,00CH,07CH,082H,0A2H,042H,0BCH,0FEH,012H,032H,052H,08CH,08CH,092H,092H,092H,062H
	DB	002H,002H,0FEH,002H,002H,07EH,080H,080H,080H,07EH,03EH,040H,080H,040H,03EH,07EH,080H,070H,080H,07EH
	DB	0C6H,028H,010H,028H,0C6H,00EH,010H,0E0H,010H,00EH,0C2H,0A2H,092H,08AH,086H,040H,020H,010H,008H,004H
	DB	002H,004H,008H,010H,020H,020H,010H,008H,004H,07EH,008H,004H,0FEH,004H,008H,020H,040H,0FEH,040H,020H
	DB	09CH,0E2H,002H,0E2H,09CH,040H,0A8H,0A8H,0A8H,0F0H,0FEH,090H,088H,088H,070H,070H,088H,088H,088H,040H
	DB	070H,088H,088H,090H,0FEH,070H,0A8H,0A8H,0A8H,030H,010H,0FCH,012H,002H,004H,018H,0A4H,0A4H,0A4H,07CH
	DB	0FEH,010H,008H,008H,0F0H,000H,088H,0FAH,080H,000H,040H,080H,088H,07AH,000H,0FEH,020H,050H,088H,000H
	DB	000H,082H,0FEH,080H,000H,0F8H,008H,030H,008H,0F0H,0F8H,010H,008H,008H,0F0H,070H,088H,088H,088H,070H
	DB	0F8H,028H,028H,028H,010H,010H,028H,028H,030H,0F8H,0F8H,010H,008H,008H,010H,090H,0A8H,0A8H,0A8H,040H
	DB	008H,07EH,088H,080H,040H,078H,080H,080H,040H,0F8H,038H,040H,080H,040H,038H,078H,080H,060H,080H,078H
	DB	088H,050H,020H,050H,088H,018H,0A0H,0A0H,0A0H,078H,088H,0C8H,0A8H,098H,088H,000H,010H,06CH,082H,000H
	DB	000H,000H,0FEH,000H,000H,000H,082H,06CH,010H,000H,010H,010H,054H,038H,010H,010H,038H,054H,010H,010H

;****************************************
;*	      Sin Table			*
;****************************************
SINTBL:	DB	07FH,08BH,098H,0A4H,0B0H,0BBH,0C6H,0D0H,0D9H,0E2H,0E9H,0EFH,0F5H,0F9H,0FCH,0FEH;Sin
	DB	0FFH,0FEH,0FCH,0F9H,0F5H,0EFH,0E9H,0E2H,0D9H,0D0H,0C6H,0BBH,0B0H,0A4H,098H,08BH
	DB	07FH,073H,066H,05AH,04EH,043H,038H,02EH,025H,01CH,015H,00FH,009H,005H,002H,000H
	DB	000H,000H,002H,005H,009H,00FH,015H,01CH,025H,02EH,038H,043H,04EH,05AH,066H,073H

	DB	000H,007H,00FH,017H,01FH,027H,02FH,037H,03FH,047H,04FH,057H,05FH,067H,06FH,077H;Sankaku
	DB	07FH,087H,08FH,097H,09FH,0A7H,0AFH,0B7H,0BFH,0C7H,0CFH,0D7H,0DFH,0E7H,0EFH,0F7H
	DB	0FFH,0F7H,0EFH,0E7H,0DFH,0D7H,0CFH,0C7H,0BFH,0B7H,0AFH,0A7H,09FH,097H,08FH,087H
	DB	07FH,077H,06FH,067H,05FH,057H,04FH,047H,03FH,037H,02FH,027H,01FH,017H,00FH,007H

	DB	000H,004H,008H,00CH,010H,014H,018H,01CH,020H,024H,028H,02CH,030H,034H,038H,03CH;Nokogiri
	DB	040H,044H,048H,04CH,050H,055H,059H,05DH,061H,065H,069H,06DH,071H,075H,079H,07DH
	DB	081H,085H,089H,08DH,091H,095H,099H,09DH,0A1H,0A5H,0AAH,0AEH,0B2H,0B6H,0BAH,0BEH
	DB	0C2H,0C6H,0CAH,0CEH,0D2H,0D6H,0DAH,0DEH,0E2H,0E6H,0EAH,0EEH,0F2H,0F6H,0FAH,0FFH

    END
